import unittest
from datetime import date

from vha_toolbox import (
    get_last_day_of_year,
    get_first_day_of_year,
    get_first_day_of_month,
    get_last_day_of_month,
    get_last_day_of_quarter,
    get_first_day_of_quarter
)


class DateUtilsTestCase(unittest.TestCase):
    def test_get_first_day_of_year(self):
        dt = date(2023, 6, 15)
        result = get_first_day_of_year(dt)
        self.assertEqual(result, date(2023, 1, 1))

    def test_get_last_day_of_year(self):
        dt = date(2023, 6, 15)
        result = get_last_day_of_year(dt)
        self.assertEqual(result, date(2023, 12, 31))

    def test_get_first_day_of_quarter(self):
        dt = date(2023, 6, 15)
        result = get_first_day_of_quarter(dt)
        self.assertEqual(result, date(2023, 4, 1))

    def test_get_last_day_of_quarter(self):
        dt = date(2023, 6, 15)
        result = get_last_day_of_quarter(dt)
        self.assertEqual(result, date(2023, 6, 30))

    def test_get_first_day_of_month(self):
        dt = date(2023, 6, 15)
        result = get_first_day_of_month(dt)
        self.assertEqual(result, date(2023, 6, 1))

    def test_get_first_day_of_month_2(self):
        dt = date(2024, 2, 10)
        result = get_first_day_of_month(dt)
        self.assertEqual(result, date(2024, 2, 1))

    def test_get_last_day_of_month(self):
        dt = date(2023, 6, 15)
        result = get_last_day_of_month(dt)
        self.assertEqual(result, date(2023, 6, 30))

    def test_get_last_day_of_month_2(self):
        dt = date(2024, 2, 10)
        result = get_last_day_of_month(dt)
        self.assertEqual(result, date(2024, 2, 29))

    def test_get_last_day_of_month_3(self):
        dt = date(2021, 2, 10)
        result = get_last_day_of_month(dt)
        self.assertEqual(result, date(2021, 2, 28))


if __name__ == '__main__':
    unittest.main()




class EAN:
    """
    A class used to represent an EAN-13.
    """
    def __init__(self, ean):
        """
        Args:
            ean (str): The EAN-13 to validate and format.

        Examples:
            >>> EAN('9783161484100')
            EAN(9783161484100)
            >>> EAN('978-3-16-148410-0')
            EAN(9783161484100)

        Raises:
            ValueError: If the EAN-13 is invalid (either format is incorrect or check digit is incorrect).
        """
        self.ean = self._sanitize_and_normalize_ean(ean)
        is_valid = self.is_valid()
        if not is_valid:
            raise ValueError("Invalid EAN, check digit is incorrect")

    def _sanitize_and_normalize_ean(self, ean):
        # Sanitize and normalize the provided EAN-13
        ean = ean.replace("-", "").replace(" ", "").lower()
        normalize = ''.join(char for char in ean if char.isdigit())
        if len(normalize) == 13:
            return normalize
        else:
            raise ValueError("Invalid EAN, format is incorrect")

    def break_down_ean(self) -> list[str]:
        """
        Breaks down an EAN-13 into its parts.

        Returns:
            list[str]: A list of strings containing the EAN-13 parts.
        """
        parts = [
            "Prefix: " + self.ean[:3],
            "Manufacturer/Product: " + self.ean[3:12],
            "Check digit: " + self.ean[12]
        ]

        return parts

    def is_valid(self) -> bool:
        """
        Validates an EAN-13 by checking if the check digit is correct.

        Returns:
            bool: True if the EAN-13 is valid, False otherwise.

        Examples:
            >>> EAN('9783161484100').is_valid()
            True
            >>> EAN('9783161484105').is_valid()
            False
        """
        check_digit = sum(int(self.ean[i]) * (1 if i % 2 == 0 else 3) for i in range(12))
        check_digit = (10 - (check_digit % 10)) % 10
        return check_digit == int(self.ean[-1])

    def format(self) -> str:
        """
        Formats an EAN-13 by adding dashes.

        Returns:
            str: The formatted EAN-13.
        """
        return self.ean

    def __str__(self):
        return self.format()

    def __repr__(self):
        return f"EAN({self.format()})"



import unittest

from vha_toolbox import EAN


class EanTestCase(unittest.TestCase):
    def test_ean_1(self):
        ean = EAN('5449000021199')
        result = ean.break_down_ean()
        self.assertEqual(result, ['Prefix: 544', 'Manufacturer/Product: 900002119', 'Check digit: 9'])
        result = ean.format()
        self.assertEqual(result, '5449000021199')


if __name__ == '__main__':
    unittest.main()



from calendar import monthrange
from datetime import date, timedelta


def get_first_day_of_year(dt: date = date.today()) -> date:
    """
    Get the first day of the year for the given date or the current date if not provided.

    Args:
        dt (date): The date for which to retrieve the first day of the year (default: current date).

    Returns:
        date: The first day of the year as a date object.

    Example:
        >>> get_first_day_of_year(date(2023, 6, 15))
        datetime.date(2023, 1, 1)
    """
    return date(dt.year, 1, 1)


def get_last_day_of_year(dt: date = date.today()) -> date:
    """
    Get the last day of the year for the given date or the current date if not provided.

    Args:
        dt (date): The date for which to retrieve the last day of the year (default: current date).

    Returns:
        date: The last day of the year as a date object.

    Example:
        >>> get_last_day_of_year(date(2023, 6, 15))
        datetime.date(2023, 12, 31)
    """
    return date(dt.year, 12, monthrange(dt.year, 12)[1])


def get_first_day_of_quarter(dt: date = date.today()) -> date:
    """
    Get the first day of the quarter for the given date or the current date if not provided.

    Args:
        dt (date): The date for which to retrieve the first day of the quarter (default: current date).

    Returns:
        date: The first day of the quarter as a date object.

    Example:
        >>> get_first_day_of_quarter(date(2023, 6, 15))
        datetime.date(2023, 4, 1)
    """
    return date(dt.year, (dt.month - 1) // 3 * 3 + 1, 1)


def get_last_day_of_quarter(dt: date = date.today()) -> date:
    """
    Get the last day of the quarter for the given date or the current date if not provided.

    Args:
        dt (date): The date for which to retrieve the last day of the quarter (default: current date).

    Returns:
        date: The last day of the quarter as a date object.

    Example:
        >>> get_last_day_of_quarter(date(2023, 6, 15))
        datetime.date(2023, 6, 30)
    """
    next_qt_yr = dt.year + (1 if dt.month > 9 else 0)
    next_qt_first_mo = (dt.month - 1) // 3 * 3 + 4
    next_qt_first_mo = 1 if next_qt_first_mo == 13 else next_qt_first_mo
    next_qt_first_dy = date(next_qt_yr, next_qt_first_mo, 1)
    return next_qt_first_dy - timedelta(days=1)


def get_first_day_of_month(dt: date = date.today()) -> date:
    """
    Get the first day of the month for the given date or the current date if not provided.

    Args:
        dt (date): The date for which to retrieve the first day of the month (default: current date).

    Returns:
        date: The first day of the month as a date object.

    Example:
        >>> get_first_day_of_month(date(2023, 6, 15))
        datetime.date(2023, 6, 1)
    """
    return date(dt.year, dt.month, 1)


def get_last_day_of_month(dt: date = date.today()) -> date:
    """
    Get the last day of the month for the given date or the current date if not provided.

    Args:
        dt (date): The date for which to retrieve the last day of the month (default: current date).

    Returns:
        date: The last day of the month as a date object.

    Example:
        >>> get_last_day_of_month(date(2023, 6, 15))
        datetime.date(2023, 6, 30)
    """
    return date(dt.year, dt.month, monthrange(dt.year, dt.month)[1])
